#!/usr/bin/env python
import strictyaml
from pathlib import Path
from rich import print

indent = lambda l: '\n'.join(map(lambda line: '\t' + line, l))

def create_induction(requires: list[str], theorem: str, implies: list[str]) -> str:
	"""
	Creates an induction in the DOT graphviz language from the given strings so that the output represents
	an ordered graph that goes from requires nodes to a theorem node that goes to all implies nodes:

	```plain
	requires[0] --------\        /----> implies[0]
	requires[1] ----> theorem --/
	...                 /       \        ...
	requires[n] -------/         \----> implies[m]
	```

	>>> create_induction(['stricte monotonie', 'ouvertitude'], 'bijection', ['racine unique'])
	'subgraph {\\n\\t"stricte monotonie"\\n\\t"ouvertitude"\\n} -> "bijection" -> subgraph {\\n\\t"racine unique"\\n}'
	"""
	to_lines = lambda l: "\n".join([f'\t"{el}"' for el in l])

	return f"""\
subgraph {{
{to_lines(requires)}
}} -> "{theorem}" -> subgraph {{
{to_lines(implies)}
}}"""


def declare_as_theorem(name: str) -> str:
	"""
	Add a line that declares a node as a theorem name (makes the node a rectangle)

	>>> declare_as_theorem('bolzano-weierstraß')
	'"bolzano-weierstraß" [shape=rect]'
	"""

	return f'"{name}" [shape=rect, color=blue]'


def generate_graph(inductions: list[str], theorem_declarations: list[str]) -> str:
	"""
	Takes a list of inductions (see `create_induction`) and theorem declarations (see `declare_as_theorem`)
	and returns a valid DOT graphviz file
	"""

	return f"""\
digraph {{
	/* styling */
	edge [arrowhead=vee]

	/* theorems */
	{indent(theorem_declarations)}

	/* inductions */
	{indent(inductions)}
}}
"""


definition = strictyaml.load(Path("./continues.yaml").read_text(encoding="utf-8"))
inductions, theorem_declarations = [], []

for name, i in definition.items():
	inductions.append(
		create_induction(requires=i["requires"], implies=i["implies"], theorem=name)
	)
	theorem_declarations.append(declare_as_theorem(name))

graph = generate_graph(inductions, theorem_declarations)
Path("./continues.dot").write_text(graph, encoding="utf-8")
